#    This software is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This software is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with This software.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

# Author: Federico Salotti
# Organization/Company: Mr_Robot_Computers_Repair 
# Country: Italy Barga(LU)
# release: 1.0 (11 April 2021 22:21)

use warnings;
use feature "say";
use Term::ANSIColor;
use utf8;

binmode STDOUT,utf8;

@field=([],[]);
$enemies=0;
$treasures=0;
$exitButtons=0;

$lv=1;
$health=100;
$hungry=0;
$gold=100;

$game_msg="Welcome To ASCII Adventure";
$last_command=" ";
$win=0;
$game_over=0;

%items=
(
#	'wood' => {'q' => 4, 'price' => 2},
#	'rope' => {'q' => 2},
#	'stone' =>{'q' => 3},
#	"hammer" => {'q' => 2, 'resistance' => 3},
#	"machete" => {'q' => 2, 'resistance' => 1},
#	"fishing pole" => {'q' =>1, 'resistance' => 2},
#	'tomato soup' => 5,
#	'fish' => 5,
);

%shop_items = ();

sub ascii_art
{

	if($_[0] eq 'backpack')
	{
		open READER,'<',"ascii_art/backpack";
		while(<READER>){chomp;say;}
		close READER;		
	}
	if($_[0] eq 'craft')
	{
		open READER,'<',"ascii_art/craft";
		while(<READER>){chomp;say;}
		close READER;		
	}
}

sub print_entity
{
	if($_[0] eq "@")
	{
		print "\033[38;05;105m\033 $_[0]";
	}
	elsif($_[0] eq "#")
	{
		print "\033[38;05;94m\033 $_[0]",color(RESET);
	}

	elsif($_[0] eq "\N{BLACK SMILING FACE}")
	{
		print "\033[38;05;106m\033 $_[0]",color(RESET);
	}

	elsif($_[0] eq "\N{END OF PROOF}")
	{
		print "\033[38;05;59m\033 $_[0]",color(RESET);
	}

	elsif($_[0] eq "\N{GREEK UPSILON WITH HOOK SYMBOL}")
	{
		print "\033[38;05;22m\033 $_[0]",color(RESET);
	}

	elsif($_[0] eq "\N{CIRCLED MINUS}")
	{
		print color(RED),$_[0],color(RESET);
	}

	elsif($_[0] eq "\N{CIRCLED PLUS}")
	{
		print color(GREEN),$_[0],color(RESET);
	}
	elsif($_[0] eq "\N{TRIPLE TILDE}")
	{
		print color(CYAN),$_[0],color(RESET);
	}
	elsif($_[0] eq "\N{RIGHTWARDS ARROW WITH HOOK}")
	{
		if($exitButtons != 0)
		{
			print color("BOLD RED"),$_[0],color(RESET);
		}
		else
		{
			print color("BOLD GREEN"),$_[0],color(RESET);
		}
	}
	elsif($_[0] eq "\N{LATIN CAPITAL LETTER H WITH STROKE}")
	{
		print "\033[38;05;100m\033 $_[0]",color(RESET);
	}
	else
	{
		print color(RESET),$_[0],color(RESET);
	}

}

sub init
{
	for(my $y=0;$y < 25; $y++)
	{
		for(my $x=0; $x < 25; $x++)
		{
			$rand_seed=int(rand(20));
			if($rand_seed < 5 && $rand_seed < 2) {$field[$y][$x]="\N{END OF PROOF}";}
			elsif($rand_seed < 5 && $rand_seed < 3) {$field[$y][$x]="\N{GREEK UPSILON WITH HOOK SYMBOL}";}
			elsif($rand_seed < 5 && $rand_seed > 3) {$field[$y][$x]="\N{TRIPLE TILDE}";}
			else {$field[$y][$x]=".";}
		}
	}
}

sub setPlayer
{
	$field[0][0]="@";
	$field[0][1]=".";
	$field[1][0]=".";
	$field[1][1]=".";
}

sub setExit
{
	$field[24][24]="\N{RIGHTWARDS ARROW WITH HOOK}";
	$field[24][23]=".";
	$field[23][24]=".";
	$field[23][23]=".";
}

sub getPlayerXY
{
	for(my $y=0; $y < 25; $y++)
	{
		for(my $x=0; $x < 25; $x++)
		{
			if($field[$y][$x] eq "@")
			{
				return $y,$x;
			}
		}
	}
}

sub moveLeft
{
	for(my $y=0; $y < 25; ++$y)
	{
		for(my $x=0; $x < 25; ++$x)
		{
			if($field[$y][$x] eq "@" && $x > 0 && $field[$y][$x-1] eq '.')
			{
				
				$field[$y][$x]=".";
				$field[$y][$x-1]="@";
				return 1;
			}
		}
	}
}
sub moveRight
{
	for(my $y=0; $y < 25; ++$y)
	{
		for(my $x=0; $x < 25; ++$x)
		{
			if($field[$y][$x] eq "@" && $x < 24 && $field[$y][$x+1] eq '.')
			{
				$field[$y][$x]=".";
				$field[$y][$x+1]="@";
				return 1;
			}
		}
	}
}
sub moveUp
{
	for(my $y=0; $y < 25; ++$y)
	{
		for(my $x=0; $x < 25; ++$x)
		{
			if($field[$y][$x] eq "@" && $y != 0 && $field[$y-1][$x] eq '.')
			{
				$field[$y][$x]=".";
				$field[$y-1][$x]="@";
				return 1;
			}
		}
	}
}
sub moveDown
{
	for(my $y=0; $y < 25; ++$y)
	{
		for(my $x=0; $x < 25; ++$x)
		{
			if($field[$y][$x] eq "@" && $y != 24 && $field[$y+1][$x] eq '.')
			{
				$field[$y][$x]=".";
				$field[$y+=1][$x]="@";
				return 1;
			}
		}
	}
}

sub generateEnemies
{
	my @y_coords=(4..22);

	my $x=int(rand(25));
	my $y=$y_coords[rand(@y_coords)];

	#say $y;

	if($field[$y][$x] ne "@")
	{
		$field[$y][$x]="\N{BLACK SMILING FACE}";
		$enemies+=1;
	}
}

sub generateExitButtons
{
	my @y_coords=(3..22);

	my $x=int(rand(25));
	my $y=$y_coords[rand(@y_coords)];

	if($field[$y][$x] eq '.')
	{
		$field[$y][$x]="\N{CIRCLED MINUS}";
		$exitButtons+=1;
	}
}
sub generateShops
{
	my @y_coords=(3..22);

	my $x=int(rand(25));
	my $y=$y_coords[rand(@y_coords)];

	if($field[$y][$x] eq '.')
	{
		$field[$y][$x]="\N{LATIN CAPITAL LETTER H WITH STROKE}";
	}
}

sub getEnemyYX
{
	for(my $y=0; $y < 25; ++$y)
	{
		for(my $x=0; $x < 25; ++$x)
		{
			if($field[$y][$x] eq "\N{BLACK SMILING FACE}")
			{
				return $y,$x;
			}
		}
	}
}

sub generateRandomItems
{
	my $quantity=int(rand(6));
	$game_msg="";

	if($quantity == 0)
	{
		$game_msg="[!] EMPTY :(";
		return;
	}
	else
	{
		$game_msg="GOOD! x$quantity NEW ITEMS! :)";
	}

	for(my $i=0; $i < $quantity; ++$i)
	{
		$item_id=int(rand(15));

		if($item_id == 1) #BREAD
		{
			$items{"bread"}{q}+=1;
			$items{"bread"}{price} = 3;
		}
		if($item_id == 2) #TOMATO SOUP
		{
			$items{"tomato soup"}{q} += 1;
			$items{"tomato soup"}{price} = 5;
		}
		if($item_id == 3) #CHICKEN
		{
			$items{"chicken"}{q} += 1;
			$items{"chicken"}{price} = 8;
		}
		if($item_id == 4) #PIZZA
		{
			$items{"pizza"}{q} += 1;
			$items{"pizza"}{price} = 10;
		}
		if($item_id == 5) #CHOCOLATE
		{
			$items{"chocolate"}{q} += 1;
			$items{"chocolate"}{price} = 15;
		}
		if($item_id == 6) #ENERGY BAR
		{
			$items{"energy bar"}{q} += 1;
			$items{"energy bar"}{price} = 25;
		}
		if($item_id == 7) #MEDIKIT
		{
			$items{"medikit"}{q} += 1;
			$items{"medikit"}{price} = 30;
		}
		if($item_id == 7) #MEDIKIT MEDIUM
		{
			$items{"medikit medium"}{q} += 1;
			$items{"medikit medium"}{price} = 50;
		}
		if($item_id == 8) #MEDIKIT LARGE
		{
			$items{"medikit large"}{q} += 1;
			$items{"medikit large"}{price} = 60;
		}

		if($item_id == 9) #STONE
		{
			$items{"stone"}{q} += 1;
			$items{"stone"}{price} = 1;
		}
		if($item_id == 10) #HAMMER
		{
			$items{hammer}{q} += 1;
			$items{hammer}{resistance} = 3;
			$items{hammer}{price} = 15;
		}
		if($item_id == 11) #ROPE
		{
			$items{"rope"}{q} +=1 ;
			$items{"rope"}{price} = 5;
		}
		if($item_id == 12) #MACHETE
		{
			$items{machete}{q}+=1;
			$items{machete}{resistance}=3;
			$items{machete}{price}=25;
		}
		if($item_id == 13) #FISHING POLE
		{
			$items{'fishing pole'}{q}+=1;
			$items{'fishing pole'}{resistance}=2;
			$items{'fishing pole'}{price}=50;
		}
		if($item_id == 14) #GOLD
		{
			$gold+=int(rand(10));
		}
	}
}
sub generateShopItems
{
	%shop_items = ();

	my $quantity=int(rand(10)) + 3;

	for(my $i=0; $i < $quantity; ++$i)
	{
		$item_id=int(rand(14));

		if($item_id == 1) #BREAD
		{
			$shop_items{"bread"}{q}+=1;
			$shop_items{"bread"}{price} = 3;
		}
		if($item_id == 2) #TOMATO SOUP
		{
			$shop_items{"tomato soup"}{q} += 1;
			$shop_items{"tomato soup"}{price} = 5;
		}
		if($item_id == 3) #CHICKEN
		{
			$shop_items{"chicken"}{q} += 1;
			$shop_items{"chicken"}{price} = 8;
		}
		if($item_id == 4) #PIZZA
		{
			$shop_items{"pizza"}{q} += 1;
			$shop_items{"pizza"}{price} = 10;
		}
		if($item_id == 5) #CHOCOLATE
		{
			$shop_items{"chocolate"}{q} += 1;
			$shop_items{"chocolate"}{price} = 15;
		}
		if($item_id == 6) #ENERGY BAR
		{
			$shop_items{"energy bar"}{q} += 1;
			$shop_items{"energy bar"}{price} = 25;
		}
		if($item_id == 7) #MEDIKIT
		{
			$shop_items{"medikit"}{q} += 1;
			$shop_items{"medikit"}{price} = 30;
		}
		if($item_id == 7) #MEDIKIT MEDIUM
		{
			$shop_items{"medikit medium"}{q} += 1;
			$shop_items{"medikit medium"}{price} = 50;
		}
		if($item_id == 8) #MEDIKIT LARGE
		{
			$shop_items{"medikit large"}{q} += 1;
			$shop_items{"medikit large"}{price} = 60;
		}

		if($item_id == 9) #STONE
		{
			$shop_items{"stone"}{q} += 1;
			$shop_items{"stone"}{price} = 1;
		}
		if($item_id == 10) #HAMMER
		{
			$shop_items{'hammer'}{q} += 1;
			$shop_items{'hammer'}{resistance} = 3;
			$shop_items{'hammer'}{price} = 15;
		}
		if($item_id == 11) #ROPE
		{
			$shop_items{"rope"}{q} +=1 ;
			$shop_items{"rope"}{price} = 5;
		}
		if($item_id == 12) #MACHETE
		{
			$shop_items{'machete'}{q}+=1;
			$shop_items{'machete'}{resistance}=3;
			$shop_items{'machete'}{price}=25;
		}
		if($item_id == 13) #FISHING POLE
		{
			$shop_items{'fishing pole'}{q}+=1;
			$shop_items{'fishing pole'}{resistance}=2;
			$shop_items{'fishing pole'}{price}=50;
		}
	}
}

sub moveEnemies
{
	for(my $y=0; $y < 25; $y++)
	{
		for(my $x=0; $x < 25; $x++)
		{
			if($field[$y][$x] eq "\N{BLACK SMILING FACE}")
			{
				my $new_x=$x;
				my $new_y=$y;
				my $move_seed=int(rand(5));

				if($move_seed == 1) {$new_x+=1;} #DX
				if($move_seed == 2) {$new_x-=1;} #SX
				if($move_seed == 3) {$new_y-=1;} #UP
				if($move_seed == 4) {$new_y+=1;} #DOWN

				if($new_x < 0) {$new_x=0;}
				if($new_y < 0) {$new_y=0;}
				if($new_x >= 24) {$new_x=24;}
				if($new_y >= 24) {$new_y=24;}

				if($field[$new_y][$new_x] ne "\N{BLACK SMILING FACE}" && $field[$new_y][$new_x] eq '.')
				{
					$field[$y][$x]=".";
					$field[$new_y][$new_x]="\N{BLACK SMILING FACE}";
				}
			}
		}
	}
}

sub checkGameOver
{

	if($health == 0) {$game_over=1;return;}

	for(my $y=1; $y < 23; ++$y)
	{
		for(my $x=1; $x < 23; ++$x)
		{
			if($field[$y][$x] eq "\N{BLACK SMILING FACE}")
			{
				if($field[$y][$x+1] eq '@'||
				   $field[$y][$x-1] eq '@'||
				   $field[$y-1][$x] eq '@'||
				   $field[$y+1][$x] eq '@') {$game_over=1;last;}
			}
		}
	}
}

sub checkWin
{
	(my $player_y, my $player_x) = getPlayerXY;

	if($exitButtons == 0 && $field[24][23] eq '@' || $field[23][24] eq '@') {$win=1;$lv+=1;return;}

	
}

sub generateTreasures
{
	my @coords = (2..23);
	my $x=$coords[rand(@coords)];
	my $y=$coords[rand(@coords)];

	if($field[$y][$x+1] eq ".")
	{
		$field[$y][$x]="#";
		$treasures+=1;
	}
}

sub drawField
{
#	system("clear");
	for(my $y=0; $y < 25; $y++)
	{
		for(my $x=0;$x < 25; $x++)
		{
			if($y == 0 && $x == 24)
			{
				print_entity $field[$y][$x];
				print"\t LV $lv";
			}
			elsif($y == 1 && $x == 24)
			{
				print_entity $field[$y][$x];
				print color(RED),"\t \N{HEAVY BLACK HEART}  $health";
			}
			elsif($y == 2 && $x == 24)
			{
				print_entity $field[$y][$x];
				if($hungry > 85) { print color(RED),"\t HUNGRY $hungry"; }
				if($hungry > 50 && $hungry < 85) { print color(YELLOW),"\t HUNGRY $hungry"; }
				if($hungry < 50 ) { print color(GREEN),"\t HUNGRY $hungry"; }
			}
			elsif($y == 3 && $x ==24)
			{
				print_entity $field[$y][$x];
				print"\033[38;5;226m \t \$GOLD \$$gold";
			}

			elsif($y == 5 && $x == 24)
			{
				print_entity $field[$y][$x];
				print "\t $game_msg";
			}
			elsif($y == 7 && $x == 24)
			{
				print_entity $field[$y][$x];
				print "\t Last command : $last_command";
			}
			elsif($y == 9 && $x == 24)
			{
				print_entity $field[$y][$x];
				(my $y, my $x) = getPlayerXY();
				print "\t Position ( x = $x | y = $y )";
			}
			elsif($y == 11 && $x == 24)
			{
				print_entity $field[$y][$x];
				if($exitButtons != 0)
					{print color(RED),"\t Exit Buttons \N{CIRCLED MINUS} x $exitButtons",color(RESET);}
				else
					{print color(GREEN),"\t Exit Buttons \N{CIRCLED MINUS} x $exitButtons",color(RESET);}
			}
			else
			{
				print_entity $field[$y][$x];
			}

		}

		print "\n";
	}
}
sub refresh()
{
	system("clear");
	drawField;
}
sub cleanItems
{
	foreach $item(keys %items)
	{
		if($item eq 'machete' || $item eq 'hammer' ||
		   $item eq 'fishing pole')
		{
			if($items{$item}{q} == 0)
			{delete $items{$item};}
		}
		else
		{
			if($items{$item}{q} == 0) {delete $items{$item};}
		}
	}
}

sub useBread
{
	$hungry-=5;
	$items{"bread"}{q}-=1;
}

sub useTomatoSoup
{
	$hungry-=15;
	$items{"tomato soup"}{q}-=1;
}

sub useChicken
{
	$hungry-=35;
	$items{"chicken"}{q}-=1;
}

sub usePizza
{
	$hungry-=50;
	$items{"pizza"}{q}-=1;
}
sub useFish
{
	$hungry-=15;
	$items{fish}{q}-=1;
}
sub useChocolate
{
	$hungry-=75;
	$items{"chocolate"}{q}-=1;
}
sub useFishSoup
{
	$hungry-=65;
	$items{'fish soup'}{q}-=1;
}
sub useEnergyBar
{
	$hungry-=100;
	$items{"energy bar"}{q}-=1;
}
sub useMedikit
{
	$health+=30;
	$items{medikit}{q}-=1;
}
sub useMedikit_medium
{
	$health+=50;
	$items{"medikit medium"}{q}-=1;
}
sub useMedikit_large
{
	$health+=70;
	$items{"medikit large"}{q}-=1;
}

sub throwStone
{
	(my $player_y, my $player_x) = getPlayerXY;

		if($field[$player_y][$player_x+2] eq "\N{BLACK SMILING FACE}") # DX +2
	{
		$field[$player_y][$player_x+2]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}


	if($field[$player_y][$player_x-2] eq "\N{BLACK SMILING FACE}") # SX -2
	{
		$field[$player_y][$player_x-2]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}


	if($field[$player_y-2][$player_x] eq "\N{BLACK SMILING FACE}") # TOP -2
	{
		$field[$player_y-2][$player_x]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}


	if($field[$player_y+2][$player_x] eq "\N{BLACK SMILING FACE}") # DOWN +2
	{
		$field[$player_y+2][$player_x]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}

	if($field[$player_y-1][$player_x+1] eq "\N{BLACK SMILING FACE}") # TOP - RIGHT
	{
		$field[$player_y-1][$player_x+1]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}

	if($field[$player_y-1][$player_x-1] eq "\N{BLACK SMILING FACE}") # TOP - LEFT
	{
		$field[$player_y-1][$player_x-1]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}

	if($field[$player_y+1][$player_x+1] eq "\N{BLACK SMILING FACE}") # DOWN - RIGHT
	{
		$field[$player_y+1][$player_x+1]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}

	if($field[$player_y+1][$player_x-1] eq "\N{BLACK SMILING FACE}") # DOWN - LEFT
	{
		$field[$player_y+1][$player_x-1]=".";
		$game_msg="GOT IT!";
		$items{"stone"}{q}-=1;
		return;
	}

sub useHammer
{
	(my $player_y, my $player_x) = getPlayerXY;

	print "[+]direction: [ a=LEFT d=RIGHT w=UP s=DOWN ]: ";

	while(<STDIN>)
	{
		chomp;

		if(/^a$/i) #LEFT
		{
			if($field[$player_y][$player_x-1] eq "\N{END OF PROOF}")
			{
				$field[$player_y][$player_x-1]=".";
				$game_msg="GOOD JOB!";
				$items{stone}{q}+=1;
				$items{stone}{price} = 1;
				if($items{hammer}{resistance} == 1)
				{
					if($items{hammer}{q} != 0)
					{
						$items{hammer}{q}-=1;
						$items{hammer}{resistance}=3;
					}
					else {}
				}
				else {$items{hammer}{resistance}-=1;}
				return;
			}
			else
			{
				$game_msg="WHY I HAVE TO USE THIS NOW?";
				return;
			}
		}
		elsif(/^d$/i) #RIGHT
		{
			if($field[$player_y][$player_x+1] eq "\N{END OF PROOF}")
			{
				$field[$player_y][$player_x+1]=".";
				$game_msg="GOOD JOB!";
				$items{stone}{q}+=1;
				$items{stone}{price} = 1;
				if($items{hammer}{resistance} == 1)
				{
					if($items{hammer}{q} != 0)
					{
						$items{hammer}{q}-=1;
						$items{hammer}{resistance}=3;
					}
					else {}
				}
				else {$items{hammer}{resistance}-=1;}

				return;
			}
			else
			{
				$game_msg="WHY I HAVE TO USE THIS NOW?";
				return;
			}
		}
		elsif(/^w$/i) #UP
		{
			if($field[$player_y-1][$player_x] eq "\N{END OF PROOF}")
			{
				$field[$player_y-1][$player_x]=".";
				$game_msg="GOOD JOB!";
				$items{stone}{q}+=1;
				$items{stone}{price} = 1;
				if($items{hammer}{resistance} == 1)
				{
					if($items{hammer}{q} != 0)
					{
						$items{hammer}{q}-=1;
						$items{hammer}{resistance}=3;
					}
					else {}
				}
				else {$items{hammer}{resistance}-=1;}

				return;
			}
			else
			{
				$game_msg="WHY I HAVE TO USE THIS NOW?";
				return;
			}
		}
		elsif(/^s$/i) #DOWN
		{
			if($field[$player_y+1][$player_x] eq "\N{END OF PROOF}")
			{
				$field[$player_y+1][$player_x]=".";
				$game_msg="GOOD JOB!";
				$items{stone}{q}+=1;
				$items{stone}{price} = 1;
				if($items{hammer}{resistance} == 1)
				{
					if($items{hammer}{q} != 0)
					{
						$items{hammer}{q}-=1;
						$items{hammer}{resistance}=3;
					}
					else {}
				}
				else {$items{hammer}{resistance}-=1;}

				return;
			}
			else
			{
				$game_msg="WHY I HAVE TO USE THIS NOW?";
				return;
			}
		}
		else # IN ANY OTHER CASE...
		{
			return;
		}

	}
}

sub useRope
{
	(my $player_y,my $player_x) = getPlayerXY;

	print "[+]direction: [ a=LEFT d=RIGHT w=UP s=DOWN ]: ";

	while(<STDIN>)
	{
		chomp;

		if(/^a$/i) #LEFT
		{
			if($field[$player_y][$player_x-1] eq "\N{END OF PROOF}" && $field[$player_y][$player_x-2] eq '.') # LEFT
			{
				$field[$player_y][$player_x]=".";
				$field[$player_y][$player_x-2]="@";
				$game_msg="HOPLA'!";
				$items{"rope"}{q}-=1;
				return;
			}
			else
			{
				$game_msg="NOT A GOOD IDEA...";
				return;
			}
		}
		elsif(/^d$/i) #RIGHT
		{
			if($field[$player_y][$player_x+1] eq "\N{END OF PROOF}" && $field[$player_y][$player_x+2] eq '.') # RIGHT
			{
				$field[$player_y][$player_x]=".";
				$field[$player_y][$player_x+2]="@";
				$game_msg="HOPLA'!";
				$items{"rope"}{q}-=1;
				return;
			}
			else
			{
				$game_msg="NOT A GOOD IDEA...";
				return;
			}
		}
		elsif(/^w$/i) #UP
		{
			if($field[$player_y-1][$player_x] eq "\N{END OF PROOF}" && $field[$player_y-2][$player_x] eq '.') # UP
			{
				$field[$player_y][$player_x]=".";
				$field[$player_y-2][$player_x]="@";
				$game_msg="HOPLA'!";
				$items{"rope"}{q}-=1;
				return;
			}
			else
			{
				$game_msg="NOT A GOOD IDEA...";
				return;
			}

		}
		elsif(/^s$/i) #DOWN
		{
			if($field[$player_y+1][$player_x] eq "\N{END OF PROOF}" && $field[$player_y+2][$player_x] eq '.') # DOWN
			{
				$field[$player_y][$player_x]=".";
				$field[$player_y+2][$player_x]="@";
				$game_msg="HOPLA'!";
				$items{"rope"}{q}-=1;
				return;
			}
			else
			{
				$game_msg="NOT A GOOD IDEA...";
				return;
			}


		}
		else{return;} # IN ANY OTHER CASE
	}

}

sub useMachete
{
	(my $player_y, my $player_x) = getPlayerXY;

	print "[+]direction: [ a=LEFT d=RIGHT w=UP s=DOWN ]: ";

	while(my $direction=<STDIN>)
	{
		chomp $direction;

		if($direction =~ /^a$/i) #LEFT
		{
			if($field[$player_y][$player_x-1] eq "\N{GREEK UPSILON WITH HOOK SYMBOL}")
			{
				$field[$player_y][$player_x-1]=".";
				$game_msg="OUT OF THE WAY!";
				if($items{machete}{resistance} == 1)
				{
					if($items{machete}{q} != 0)
					{
						$items{machete}{q}-=1;
						$items{machete}{resistance}=3;
					}
					else {$items{machete}{q}-=1;}
				}
				else{$items{machete}{resistance}-=1;}

				if(int(rand(11)) > 5)
				{
					$items{wood}{q}+=1;
					$items{wood}{price}=2;
				}
				else
				{
					$items{vine}{q}+=1;
					$items{vine}{price}=2;
				}
				last;


			}
			else
			{
				$game_msg="NOTHING TO CUT";
				return;
			}
		}
		elsif($direction =~ /^d$/i) #RIGHT
		{
			if($field[$player_y][$player_x+1] eq "\N{GREEK UPSILON WITH HOOK SYMBOL}")
			{
				$field[$player_y][$player_x+1]=".";
				$game_msg="OUT OF THE WAY!";
				if($items{machete}{resistance} == 1)
				{
					if($items{machete}{q} != 0)
					{
						$items{machete}{q}-=1;
						$items{machete}{resistance}=3;
					}
					else {$items{machete}{q}-=1;}
				}
				else{$items{machete}{resistance}-=1;}



				if(int(rand(11)) > 5)
				{
					$items{wood}{q}+=1;
					$items{wood}{price}=2;
				}
				else
				{
					$items{vine}{q}+=1;
					$items{vine}{price}=1;
				}
				last;

			}
			else
			{
				$game_msg="NOTHING TO CUT";
				return;
			}
		}
		elsif($direction =~ /^w$/i) #UP
		{
			if($field[$player_y-1][$player_x] eq "\N{GREEK UPSILON WITH HOOK SYMBOL}")
			{
				$field[$player_y-1][$player_x]=".";
				$game_msg="OUT OF THE WAY!";
				if($items{machete}{resistance} == 1)
				{
					if($items{machete}{q} != 0)
					{
						$items{machete}{q}-=1;
						$items{machete}{resistance}=3;
					}
					else {$items{machete}{q}-=1;}
				}
				else{$items{machete}{resistance}-=1;}



				if(int(rand(11)) > 5)
				{
					$items{wood}{q}+=1;
					$items{wood}{price}=2;
				}
				else 
				{
					$items{vine}{q}+=1;
					$items{vine}{price}=1;
				}
				last;

			}
			else
			{
				$game_msg="NOTHING TO CUT";
				return;
			}
		}
		elsif($direction =~ /^s$/i) #DOWN
		{
			if($field[$player_y+1][$player_x] eq "\N{GREEK UPSILON WITH HOOK SYMBOL}")
			{
				$field[$player_y+1][$player_x]=".";
				$game_msg="OUT OF THE WAY!";
				if($items{machete}{resistance} == 1)
				{
					if($items{machete}{q} != 0)
					{
						$items{machete}{q}-=1;
						$items{machete}{resistance}=3;
					}
					else {$items{machete}{q}-=1;}
				}
				else{$items{machete}{resistance}-=1;}



				if(int(rand(11)) > 5)
				{
					$items{wood}{q}+=1;
					$items{wood}{price}=2;
				}
				else
				{
					$items{vine}{q}+=1;
					$items{vine}{price}=1;
				}
				last;
			}
			else
			{
				$game_msg="NOTHING TO CUT";
				return;
			}
		}
		else {return;} # IN ANY OTHER CASE...
	}
}

sub useFishingPole
{
	(my $player_y,my $player_x) = getPlayerXY;

	if($field[$player_y][$player_x+1] eq "\N{TRIPLE TILDE}") #RIGHT
	{
		if(int(rand(10))%3 == 0)
		{
			$game_msg="BINGO!";
			$items{fish}{q}+=1;
			$items{fish}{price}=10;
#			return ;
		}
		else
		{
			$game_msg="DAMN...";
#			return;
		}

		if($items{'fishing pole'}{resistance} == 1)
		{
			if($items{'fishing pole'}{q} != 0)
			{
				$items{'fishing pole'}{q}-=1;
				$items{'fishing pole'}{resistance}=2;
			}
			else{}
		}
		else
		{
			$items{'fishing pole'}{resistance}-=1;
		}
		return;
	}
	if($field[$player_y][$player_x-1] eq "\N{TRIPLE TILDE}") #LEFT
	{
		if(int(rand(10))%3 == 0)
		{
			$game_msg="BINGO!";
			$items{fish}{q}+=1;
			$items{fish}{price} = 10;
#			return ;
		}
		else
		{
			$game_msg="DAMN...";
#			return;
		}

		if($items{'fishing pole'}{resistance} == 1)
		{
			if($items{'fishing pole'}{q} != 0)
			{
				$items{'fishing pole'}{q}-=1;
				$items{'fishing pole'}{resistance}=2;
			}
			else{}
		}
		else
		{
			$items{'fishing pole'}{resistance}-=1;
		}
		return;

	}
	if($field[$player_y-1][$player_x]) #UP
	{
		if(int(rand(10))%3 == 0)
		{
			$game_msg="BINGO!";
			$items{fish}{q}+=1;
			$items{fish}{price} = 10;
#			return ;
		}
		else
		{
			$game_msg="DAMN...";
#			return;
		}
		if($items{'fishing pole'}{resistance} == 1)
		{
			if($items{'fishing pole'}{q} != 0)
			{
				$items{'fishing pole'}{q}-=1;
				$items{'fishing pole'}{resistance}=2;
			}
			else{}
		}
		else
		{
			$items{'fishing pole'}{resistance}-=1;
		}
		return;

	}
	if($field[$player_y+1][$player_x]) #DOWN
	{
		if(int(rand(10))%3 == 0)
		{
			$game_msg="BINGO!";
			$items{fish}{q}+=1;
			$items{fish}{price}=10;
#			return ;
		}
		else
		{
			$game_msg="DAMN...";
#			return;
		}
		if($items{'fishing pole'}{resistance} == 1)
		{
			if($items{'fishing pole'}{q} != 0)
			{
				$items{'fishing pole'}{q}-=1;
				$items{'fishing pole'}{resistance}=2;
			}
			else{}
		}
		else
		{
			$items{'fishing pole'}{resistance}-=1;
		}
		return;

	}
	else
	{
		$game_msg="NO FISHING AREA HERE...";
		return;
	}

}



}

sub craftItem
{
	my $user_choice="";

	if(keys %items)
	{
		foreach $item (keys %items)
		{
			if($item eq 'hammer' || $item eq 'machete')
			{
				if($items{$item}{resistance} == 3) {$resistance=100}				
				if($items{$item}{resistance} == 2) {$resistance=65}				
				if($items{$item}{resistance} == 1) {$resistance=30}				
				say "$item x$items{$item}{q} $resistance%";
			}
			elsif($item eq 'fishing pole')
			{
				if($items{$item}{resistance} == 2) {$resistance=100}
				if($items{$item}{resistance} == 1) {$resistance=50}
				say "$item x$items{$item}{q} $resistance%";
			}
			else {say "$item x$items{$item}{q}";}
		}
		while(<STDIN>)
		{
			chomp;

			if(/^show$/i)
			{
				say "> hammer = wood x1 + stone x1";
				say "> rope = vine x2";
				say "> fish soup = tomato soup x1 + fish x2";
				say "> fishing pole= wood x2 + rope x1";
			}
			elsif(/^(wood|stone) ?(wood|stone)$/)
			{
				if($items{wood}{q} >= 1 && $items{stone}{q} >= 1)
				{
					print "[?] build hammer? [y/n]";
					$user_choice=<STDIN>;
					chomp $user_choice;
					$user_choice="n" unless $user_choice;

					if($user_choice =~ /^y$/i)
					{
						$items{wood}{q} -= 1;
						$items{stone}{q} -= 1;

						$items{hammer}{q}+=1;
						$items{hammer}{resistance}=3;
						$items{hammer}{price}=15;
						return;
					}
					else {return;}
				}
				else
				{
					$game_msg="[!] I CAN'T BUILD IT...";
					return;
				}
			}
			elsif(/^vine ?vine$/)
			{
				if($items{vine}{q} >= 2)
				{
					print "[?] build rope? [y/n]";
					$user_choice=<STDIN>;
					chomp $user_choice;
					$user_choice="n" unless $user_choice;

					if($user_choice =~ /^y$/i)
					{
						$items{vine}{q}-=2;
						$items{rope}{q}+=1;
						$items{rope}{price}=5;
						return;
					}
					else {return;}
				}
			}
			elsif(/^(tomato ?soup|fish) ?(tomato ?soup|fish)$/i)
			{
				if($items{'tomato soup'}{q} >= 1 && $items{'fish'}{q} >= 2)
				{
					print "[?] cook fish soup? [y/n]";
					$user_choice=<STDIN>;
					chomp $user_choice;
					$user_choice="n" unless $user_choice;

					if($user_choice =~ /^y$/i)
					{
						$items{'tomato soup'}{q}-=1;
						$items{'fish'}{q}-=2;
						$items{'fish soup'}{q}+=1;
						$items{'fish soup'}{price}=30;
						return;
					}
					else {return;}
				}
				else
				{
					$game_msg="[!] I CAN'T COOK IT...";
					return;
				}
			}

			elsif(/^(wood|rope) ?(wood|rope)$/)
			{
				if($items{wood}{q} >=2 && $items{rope}{q} >= 1)
				{
					print "[?] build fishing pole? [y/n]";
					$user_choice=<STDIN>;
					chomp $user_choice;
					$user_choice="n" unless $user_choice;

					if($user_choice =~ /^y$/i)
					{
						$items{wood}{q}-=2;
						$items{rope}{q}-=1;

						$items{'fishing pole'}{q}+=1;
						$items{'fishing pole'}{resistance}=2;
						$items{'fishing pole'}{price}=50;
						return;
					}
					else {return;}
					
				}
				else
				{
					$game_msg="[!] I CAN'T BUILD IT...";
					return;
				}
			}

			else {return;}
		}
	}
	else
	{
		$game_msg="NOTHING TO DO...";
		return;
	}
}

sub shop
{
	(my $player_y, ,my $player_x) = getPlayerXY();

	if($field[$player_y][$player_x+1] eq "\N{LATIN CAPITAL LETTER H WITH STROKE}" || #DX
	   $field[$player_y][$player_x-1] eq "\N{LATIN CAPITAL LETTER H WITH STROKE}" || #SX
	   $field[$player_y-1][$player_x] eq "\N{LATIN CAPITAL LETTER H WITH STROKE}" || #UP
	   $field[$player_y+1][$player_x] eq "\N{LATIN CAPITAL LETTER H WITH STROKE}"    #DOWN
	)

	{
		$game_msg="[+] Welcome to my Shop!";
		generateShopItems;
		refresh;

		print "action: ";

		while(<STDIN>)
		{
			chomp;

			if(/^buy$/ig)
			{
				$game_msg = "[*] Look at this...";
				refresh;
				BUY_MENU:
				foreach $element (keys %shop_items)
				{
					if($shop_items{$element}{q} > 0)
					{
						print "[*] $element x $shop_items{$element}{q} - \$$shop_items{$element}{price}\n";
					}
					else
					{
						delete $shop_items{$element};
					}
				}

				print "[+] action: ";
	
				while(<STDIN>)
				{
					chomp;
					if(/^(sell)$/ig) {goto SELL;}
					elsif(/^(exit|bye)$/ig) {goto EXIT_SHOP;}
					elsif(/^buy (.*)$/ig)
					{
						if(exists($shop_items{$1}))
						{
							if($gold >= $shop_items{$1}{price})
							{
								$shop_items{$1}{q} -= 1;
								$gold -= $shop_items{$1}{price};
								$items{$1}{q} += 1;
								$items{$1}{price} = $shop_items{$1}{price};
								if(lc($1) eq "hammer" || lc($1) eq "machete" || lc($1) eq "fishing pole")
								{
									if(!exists($items{$1}{resistance}))
									{
										$items{$1}{resistance} = $shop_items{$1}{resistance};
									}
					
								}
								$game_msg = "[*] You purchased $1";
								refresh;
								goto BUY_MENU;
							}
							else
							{
								$game_msg = "[!] You don't have enough money...";
								refresh;
								goto BUY_MENU; 
							}
						}
						else {goto BUY_MENU;}
					}
					else {goto BUY_MENU;}
				}
			}
			elsif(/^sell$/ig)
			{
				SELL:
				$game_msg = "[*] Mmmm... let's see...";
				refresh;

				SELL_MENU:
				cleanItems;
				foreach $item (keys(%items))
				{
					print "$item x$items{$item}{q} \$$items{$item}{price}\n";
				}
				print "[+] action: ";

				while(<STDIN>)
				{
					chomp;
					if(/^sell (.*)$/ig)
					{
						foreach $item(keys(%items))
						{
							if($item eq $1)
							{
								$items{$item}{q} -= 1;
								$gold += $items{$item}{price};
								refresh;
								goto SELL_MENU;
							}
						}
						goto SELL_MENU;
					}
					elsif(/^buy$/ig)
					{
						goto BUY_MENU;
					}
					elsif(/^(exit|bye)$/ig)
					{
						goto EXIT_SHOP;
					}
					else
					{
						goto SELL_MENU;
					}
				}
			}
			elsif(/^(exit|bye)$/ig)
			{
				EXIT_SHOP:
				$game_msg = "[*] Bye!";
				if($field[$player_y][$player_x + 1]) {$field[$player_y][$player_x + 1] = "."}
				if($field[$player_y][$player_x - 1]) {$field[$player_y][$player_x - 1] = "."}
				if($field[$player_y + 1][$player_x]) {$field[$player_y + 1][$player_x] = "."}
				if($field[$player_y - 1][$player_x]) {$field[$player_y - 1][$player_x] = "."}
				last;
			}
			else
			{goto BUY_MENU;}
		}
	}
	
}

sub isValidCommand
{
	my @commands=qw(a w d s open 'open items' craft shop);

	if(grep /^$_[1]$/i, @commands) {return 1;}
	return 0;

}

START:

init;
setPlayer;
setExit;
while($enemies < 7) {generateEnemies};
while($exitButtons < int(rand(5))) {generateExitButtons;}
while($treasures < int(rand(6))) {generateTreasures;}
while($treasures < int(rand(3))) {generateShops;}
#validateField;
drawField;

MAIN_LOOP:
print "[+] action : ";

while(<STDIN>)
{
	checkWin;
	if($win)
	{
		$win=0;
		$enemies=0;
		$treasures=0;
		$exitButtons=0;
		goto START;
	}
	checkGameOver;
	if(!$game_over)
	{
		chomp;
	COMMAND_EXECUTION:
		if(/^ a  $/ix)
		{
			if(moveLeft)
			{
				if($hungry < 100) {$hungry+=1;}
				else {$health-=2;}
			}
			$last_command=$_;

		}
		elsif(/^ d  $/ix)
		{
			if(moveRight)
			{
				if($hungry < 100) {$hungry+=1;}
				else {$health-=2;}
			}
			$last_command=$_;

		}
		elsif(/^ w  $/ix)
		{
			if(moveUp)
			{
				if($hungry < 100) {$hungry+=1;}
				else {$health-=2;}
			}
			$last_command=$_;

		}
		elsif(/^ s  $/ix)
		{
			if(moveDown)
			{
				if($hungry < 100) {$hungry+=1;}
				else {$health-=2;}
			}
			$last_command=$_;

		}

		elsif(/\bopen items\b/i)
		{
			ascii_art('backpack');
			if(!keys %items)
			{
				print "=== [ ITEMS ] ===\n";
				print "[!] EMPTY\n ";
			}
			else
			{
				@items=keys %items;

				foreach $item (@items)
				{
					if($item eq 'machete' || $item eq 'hammer')
					{
						if($items{$item}{resistance} == 3)
						{say "[*] $item x$items{$item}{q} %100";}

						if($items{$item}{resistance} == 2)
						{say "[*] $item x$items{$item}{q} %65";}

						if($items{$item}{resistance} == 1)
						{say "[*] $item x$items{$item}{q} %30";}
					}

					elsif($item eq 'fishing pole')
					{
						if($items{$item}{resistance} == 2)
						{say "[*] $item x$items{$item}{q} %100";}

						if($items{$item}{resistance} == 1)
						{say "[*] $item x$items{$item}{q} %50";}
					}

					else {say "[*] $item x$items{$item}{q}";}
				}
			}

			print "[+] select : ";

			while(<STDIN>)
			{
				chomp;


				if(keys %items)
				{
					if($items{$_} != 0)
					{
						if($items{bread} && /\bbread\b/i) #BREAD EFFECT
						{
							useBread;
						}
						if($items{"tomato soup"} && /^tomato soup$/i) #TOMATO SOUP EFFECT
						{
							useTomatoSoup;
						}
						if($items{chicken} && /\bchicken\b/i) #CHICKEN EFFECT
						{
							useChicken;
						}
						if($items{pizza} && /\bpizza\b/i) #PIZZA EFFECT
						{
							usePizza;
						}
						if($items{chocolate} && /\bchocolate\b/i) #CHOCOLATE EFFECT
						{
							useChocolate;
						}
						if($items{"energy bar"} && /^energy bar$/i) #ENERGY BAR EFFECT
						{
							useEnergyBar;
						}
						if($items{medikit} && /\bmedikit\b/i) #MEDIKIT EFFECT
						{
							useMedikit;
						}
						if($items{"medikit medium"} && /^medikit medium$/i) #MEDIKIT MEDIUM EFFECT
						{
							useMedikit_medium;
						}
						if($items{"medikit large"} && /^medikit large$/i) #MEDIKIT LARGE EFFECT
						{
							useMedikit_large;
						}
						if($items{stone} && /\bstone\b/i) #STONE EFFECT
						{
							throwStone;	
						}

						if($items{hammer} && /^hammer$/i) #HAMMER EFFECT
						{
							useHammer;						
						}
						if($items{rope} && /^rope$/i) #ROPE EFFECT
						{
							useRope;
						}
						if($items{machete} && /^machete$/) #MACHETE EFFECT
						{
							useMachete;
						}
						if($items{"fishing pole"} && /^fishing pole$/) #FISHING POLE EFFECT
						{
							useFishingPole;
						}
						if($items{fish} && /^fish$/) #FISH EFFECT
						{
							useFish;
						}
						if($items{'fish soup'} && /^fish ?soup$/i)
						{
							useFishSoup;
						}
					}
				}

				if(/\b(quit|exit)\b/i) {last;}
				else {last;}

			}
#			$last_command=$_;
		}

		elsif(/\bopen\b/i)
		{
			($player_y,$player_x)=getPlayerXY;

			if($field[$player_y][$player_x+1] eq "#")
			{
				$game_msg="[!] YOU FIND A TREASURE!";
				$field[$player_y][$player_x+1]=".";
				generateRandomItems;
			}
			elsif($field[$player_y][$player_x-1] eq "#")
			{
				$game_msg="[!] YOU FIND A TREASURE!";
				$field[$player_y][$player_x-1]=".";
				generateRandomItems;
			}
			elsif($field[$player_y+1][$player_x] eq "#")
			{
				$game_msg="[!] YOU FIND A TREASURE!";
				$field[$player_y+1][$player_x]=".";
				generateRandomItems;
			}
			elsif($field[$player_y-1][$player_x] eq "#")
			{
				$game_msg="[!] YOU FIND A TREASURE!";
				$field[$player_y-1][$player_x]=".";
				generateRandomItems;
			}
			else
			{
				$game_msg="[!] NOTHING TO OPEN...";
			}
			$last_command=$_;
		}
		elsif(/^press$/i)
		{
			(my $y, my $x) = getPlayerXY;

			if($field[$y][$x+1] eq "\N{CIRCLED MINUS}")
			{
				$field[$y][$x+1]="\N{CIRCLED PLUS}";
				$exitButtons-=1;
			}
			if($field[$y][$x-1] eq "\N{CIRCLED MINUS}")
			{
				$field[$y][$x-1]="\N{CIRCLED PLUS}";
				$exitButtons-=1;
			}
			if($field[$y-1][$x] eq "\N{CIRCLED MINUS}")
			{
				$field[$y-1][$x]="\N{CIRCLED PLUS}";
				$exitButtons-=1;
			}
			if($field[$y+1][$x] eq "\N{CIRCLED MINUS}")
			{
				$field[$y+1][$x]="\N{CIRCLED PLUS}";
				$exitButtons-=1;
			}
			$last_command=$_;

		}
		elsif(/^craft$/i)
		{
			ascii_art('craft');
			craftItem;
			$last_command=$_;
		}
		elsif(/^shop$/)
		{
			shop;

		}
		else
		{
			if($_ eq undef)
			{
				$_=$last_command;
				goto COMMAND_EXECUTION;
			}
			else
			{
				refresh;
				$game_msg="[!] INVALID COMMAND";
				goto MAIN_LOOP;
			}
		}

		if(keys %items) {cleanItems;}
		moveEnemies;
		refresh;
		print "[+] action : ";
	}
	else
	{
		say "GAME OVER";
		last;
	}

}
